package OpenStack::MetaAPI::API::Specs::Default;

use strict;
use warnings;

use Moo;

with 'OpenStack::MetaAPI::API::Specs::Roles::Service';


1;

=pod

=encoding UTF-8

=head1 NAME

OpenStack::MetaAPI::API::Specs::Default

=head1 VERSION

version 0.002

This is a void specs so we do not have to define all of them for now

=head1 AUTHOR

Nicolas R <atoomic@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2019 by cPanel, Inc.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

__DATA__